/*************************************************
 **                                             **
 **   softc.h                                   **
 **                                             **
 **       SoftC common library header           **
 **                                             **
 **          Copyright (C) 1988-1992 by         **
 **               SoftC, Ltd.                   **
 **               16820 Third Street NE         **
 **               Anoka, MN 55304               **
 **               (612) 434-6968                **
 **                                             **
 **             All rights reserved.            **
 *************************************************/


#ifndef __SOFTC_H__
#define __SOFTC_H__

/**********
 *
 *  Function Return Codes
 *
 **********/



#define SC_EXISTS     11    /* WARNING - object already exists */
#define SC_KEYPRSNT   11
#define SC_NOFUNC     10    /* WARNING - function not supported */
#define SC_MEMWRN     9     /* WARNING - partial memory allocation */
#define SC_NOTBFRD    8     /* WARNING - file I/O not buffered */
#define SC_FILENGTH   7     /* WARNING - file length is incorrect */
#define SC_FLDROUND   6     /* WARNING - numeric field rounded */
#define SC_FLDTRUNC   5     /* WARNING - data field truncated */
#define SC_NOFIND     4     /* WARNING - could not find key in index file */
#define SC_END        3     /* WARNING - no more keys */
#define SC_EMPTY      2     /* WARNING - file is empty */
#define SC_DELREC     1     /* WARNING - record read is marked deleted */
#define SC_SUCCESS    0     /* successful function return code */
#define SC_WRTFAIL    -1    /* ERROR - file write failure */
#define SC_RDFAIL     -2    /* ERROR - file read failure */
#define SC_MEMERR     -3    /* ERROR - memory allocation error */
#define SC_SKFAIL     -4    /* ERROR - file pointer reposition failed */
#define SC_NOFILE     -5    /* ERROR - file not found */
#define SC_FILBAD     -6    /* ERROR - file corrupted */
#define SC_BADEXPR    -7    /* ERROR - bad user specified key expression */
#define SC_NOHNDL     -8    /* ERROR - no handles available */
#define SC_NOPGS      -9    /* ERROR - no index pages loaded */
#define SC_BADPG      -10   /* ERROR - index page was not loaded */
#define SC_CLOSFAIL   -11   /* ERROR - file close failure */
#define SC_BADCMD     -12   /* ERROR - invalid command */
#define SC_BADHNDL    -13   /* ERROR - invalid handle number */
#define SC_BADFNAME   -14   /* ERROR - invalid filename */
#define SC_BADDATE    -15   /* ERROR - invalid date */
#define SC_BADTIME    -16   /* ERROR - invalid time */
#define SC_NODBT      -17   /* ERROR - file not in memo format */
#define SC_NOFPT      -17
#define SC_DBFVERS    -18   /* ERROR - invalid file version */
#define SC_MDXVERS    -18
#define SC_DBTVERS    -18
#define SC_NTXVERS    -18
#define SC_NDXVERS    -18
#define SC_IDXVERS    -18
#define SC_FPTVERS    -18
#define SC_DBFHLEN    -19   /* ERROR - file header length error */
#define SC_MDXHLEN    -19
#define SC_DBFDATE    -20   /* ERROR - last file change date in error */
#define SC_MDXDATE    -20
#define SC_NULLPARM   -21   /* ERROR - parameter address NULL */
#define SC_BADKEYT    -22   /* ERROR - invalid key type */
#define SC_KEYLEN     -23   /* ERROR - invalid key length */
#define SC_ITEMLEN    -24   /* ERROR - item length incorrect */
#define SC_BADROOT    -25   /* ERROR - invalid root page */
#define SC_MAXKEYS    -26   /* ERROR - bad maximum number of keys per page */
#define SC_FLDCNT     -27   /* ERROR - invalid number of fields */
#define SC_BADFLDN    -28   /* ERROR - field name invalid */
#define SC_FLDLEN     -29   /* ERROR - bad field length */
#define SC_DECPL      -30   /* ERROR - decimal places parameter invalid */
#define SC_BADFLDT    -31   /* ERROR - invalid field type */
#define SC_RECLEN     -32   /* ERROR - invalid record length */
#define SC_BADDATA    -33   /* ERROR - bad data */
#define SC_LINELEN    -34   /* ERROR - memo soft line length invalid */
#define SC_MDXFLAG    -35   /* ERROR - MDX flag in DBF file invalid */
#define SC_READOLY    -36   /* ERROR - file open for reading only */
#define SC_LCKVIOL    -37   /* ERROR - file locking violation */
#define SC_LCKBOVR    -38   /* ERROR - sharing buffer overflow */
#define SC_NOPATH     -39   /* ERROR - path not found */
#define SC_ACCDEN     -40   /* ERROR - access to file denied */
#define SC_BADACC     -41   /* ERROR - invalid access code */
#define SC_NOTLCKD    -42   /* ERROR - file must be locked first */
#define SC_NEWDEV     -43   /* ERROR - diskette changed */
#define SC_MINKEYS    -44   /* ERROR - bad minimum number of keys per page */
#define SC_FILSOPEN   -45   /* ERROR - some files remain open */
#define SC_OPENFAIL   -46   /* ERROR - could not open the file */
#define SC_FLSHFAIL   -47   /* ERROR - flush to disk failure */
#define SC_BADTAG     -48   /* ERROR - invalid tag handle */
#define SC_BLKSZ      -49   /* ERROR - invalid page size in blocks */
#define SC_BADTNAME   -50   /* ERROR - invalid tag name */
#define SC_BLKADR     -51   /* ERROR - invalid page offset adder */
#define SC_MAXTAGS    -52   /* ERROR - invalid maximum number of tag table elements */
#define SC_TBLELEN    -53   /* ERROR - bad tag table element length */
#define SC_TAGCNT     -54   /* ERROR - invalid tag count */
#define SC_KEYFORM    -55   /* ERROR - unknown key format switches */
#define SC_UNSWITCH   -56   /* ERROR - unknown switch error */
#define SC_TAGOPEN    -57   /* ERROR - tag in use */
#define SC_MLCKERR    -58   /* ERROR - Windows GlobalLock Failed */
#define SC_TASKERR    -59   /* ERROR - Windows Task lookup Failed */



/**********
 *
 *  Common Defines
 *
 **********/



/*
 *  Enable ANSI prototypes for C++ and ANSI C compilers
 */
#ifndef VMS
#define __ANSI_PROTOTYPES__
#endif

#define SC_TRUE 1
#define SC_FALSE 0
#define CONST const

#ifdef __SCO_XS5__
#undef CONST
#define CONST
#endif

#ifdef __SPARC__
#undef CONST
#define CONST
#endif

#ifdef __ULTRIX__
#undef CONST
#define CONST
#endif

#ifdef THINK_C
#undef CONST
#define CONST
#endif

#ifdef _WINDOWS
#define _Windows
#endif

#ifdef _Windows
#ifdef __DLL__
#define PROTOS FAR PASCAL _export
#else
#define PROTOS FAR PASCAL
#endif
#else
#define PROTOS
#endif

/*
 *  variable type definitions
 */

#ifndef F_PTR
#ifdef _Windows
#define F_PTR far *
#else
#define F_PTR *
#endif
#endif

#ifndef OS2DEF_INCLUDED
#define CHAR char
#define UCHAR unsigned char
#define SHORT short
#define USHORT unsigned short
#define INT int
#define UINT unsigned int
#define LONG long
#define ULONG unsigned long
#define VOID void
#endif

#define SINGLE float

#ifdef THINK_C
#define DOUBLE short double
#else
#define DOUBLE double
#endif


/* scdikfind, scdnkfind, scdckfind, scdmkfind definitions */
#define SC_FIRST 1                  /* find first key */
#define SC_EXACT 2                  /* find exact key (uses record number) */
#define SC_WILDCARD 4               /* enable use of wildcards in search */

/* scdcopen, scddopen, scdiopen, scdmopen, scdnopen, scdtopen, scdwopen */
#define SC_RDWR 0                   /* read/write access (default) */
#define SC_RDONLY 0x100             /* read only access */
#define SC_BUFFER 0                 /* buffer disk writes in memory (default) */
#define SC_FLUSH 0x200              /* force all writes to disk */
#define SC_EXCLUDE 0                /* open file for exclusive use (default) */
#define SC_SHARED 0x400             /* open file for shared use */
#define SC_MANLOCK 0                /* apply locks manually */
#define SC_AUTOLOCK 0x800           /* apply locks automatically */
#define SC_LAN SC_FLUSH|SC_SHARED   /* typical (shared and flushed) */

/* scdcbfrsz, scddbfrsz, scdibfrsz, scdnbfrsz */
#define SC_SETSZ 0                  /* set buffer size */
#define SC_GETSZ 1                  /* get current buffer size */

/* scddrput, scdtrput, scdwrput */
#define SC_UPDATE 0x100             /* update existing record */
#define SC_ADD 0                    /* append record at end of file */

/* scdccreate, scdicreate, scdncreate, scdmtnew */
#define SC_PINDEX 0x200		    /* production index (MDX & CDX only) */
#define SC_DESCEND 0x100            /* descending key order */
#define SC_UNIQUE 0x80              /* unique index keys */
#define SC_CKEY 'C'                 /* character type key */
#define SC_DKEY 'D'                 /* date type key */
#define SC_LKEY 'L'                 /* logical key type */
#define SC_NKEY 'N'                 /* numeric type key */

/* SoftC extended index file key types (scdncreate) */
#define SC_DBLKEY 1                 /* DOUBLE key */
#define SC_FLTKEY 2                 /* SINGLE key */
#define SC_ULNGKEY 3                /* ULONG key */
#define SC_LNGKEY 4                 /* LONG key */
#define SC_USHTKEY 5                /* USHORT key */
#define SC_SHTKEY 6                 /* SHORT key */

/* scdinit */
#define SC_USEXHNDL 1               /* allow more than 20 open files */

/* SoftC extended data file field types */
#define SC_SHORT_FLD 1              /* signed short integer (16 bits) */
#define SC_USHORT_FLD 2             /* unsigned short integer */
#define SC_LONG_FLD 3               /* signed long integer (32 bits) */
#define SC_ULONG_FLD 4              /* unsigned long integer */
#define SC_FLOAT_FLD 11             /* IEEE single real (32 bits) */
#define SC_MSFLT_FLD 12             /* Microsoft floating point (old) */
#define SC_DOUBLE_FLD 13            /* IEEE double real (64 bits) */
#define SC_MSDBL_FLD 14             /* Microsoft double (old) */

/* file style flag */
#define SC_DB3    1                 /* dBase3 */
#define SC_DB4    2                 /* dBase4 */
#define SC_FB     3                 /* FoxBase Plus */
#define SC_FP1    4                 /* FoxPro 1.0 */
#define SC_CLIP87 5                 /* Clipper Summer 87 */
#define SC_CLIP5  6                 /* Clipper 5.0 */
#define SC_DBXL   7                 /* dBXL & QuickSilver */
#define SC_FP2    8                 /* FoxPro 2.0 */
#define SC_OMNIS  9                 /* OMNIS */
#define SC_DBL2 0x42                /* SoftC unique */
#define SC_CLIP   SC_CLIP87         /* Clipper */


/* SC_???INFO lock flags */
#define SC_DRECLCK 0x01             /* record lock active */
#define SC_DFILLCK 0x02             /* file lock active */
#define SC_DHDRLCK 0x03             /* file header lock active */






/**********
 *
 *  Type Definitions
 *
 **********/




typedef struct {  /* file open bit fields */
  UINT isreadonly:1;    /* read only    - or - read/write (default) */
  UINT isshareable:1;   /* shared       - or - single (default) */
  UINT isunbuffered:1;  /* no buffering - or - auto buffer (default) */
  UINT filler:13;
} SC_FLAGS;


#ifdef _Windows
/* use a structure for Windows 3.0 */
typedef struct {      /* character buffer */
  HANDLE h;           /* allocated memory handle */
  CHAR F_PTR a;       /* pointer to allocated memory */
} SC_BFR;
#else
/* use a union for everyone else */
typedef union {
  CHAR F_PTR h;       /* address of allocated memory */
  CHAR F_PTR a;       /* pointer to allocated memory */
} SC_BFR;
#endif




/**********
 *
 *  Program Control Prototypes
 *
 **********/




#ifdef __ANSI_PROTOTYPES__
#ifdef __cplusplus
extern "C" {
#endif
INT PROTOS scdinit(    INT,
		       INT );
VOID PROTOS scdterm(   VOID );
#ifdef _Windows
VOID PROTOS scdqclose( VOID );
#else
#define scdqclose
#endif
INT PROTOS scversion(  CHAR F_PTR);
#ifdef __cplusplus
}
#endif
#endif



/**********
 *
 *  Error Handling Prototypes
 *
 **********/



#define sceclr() scescode(SC_SUCCESS)

#ifdef __ANSI_PROTOTYPES__
#ifdef __cplusplus
extern "C" {
#endif
CHAR F_PTR PROTOS scemsg( VOID );

INT PROTOS scecode(    VOID );           /* get sc_code */

INT PROTOS scescode(   INT );            /* set sc_code */

INT PROTOS sceslog(    INT,              /* enable/disable error logging */
		       CHAR F_PTR);      /* error log filename */

#ifdef __cplusplus
}
#endif
#endif



/**********
 *
 *  Memory and String Function Definitions
 *
 **********/



#ifdef _Windows
#define ULEN ULONG
#define M_HNDL HANDLE
#define scslen(a)         _fstrlen(a)
#define scscmp(a,b)       _fstrcmp(a,b)
#define scsicmp(a,b)      _fstricmp(a,b)
#define scsrchr(a,b)      _fstrrchr(a,b)
#define scsstr(a,b)       _fstrstr(a,b)
#define scmcpy(a,b,c)     _fmemcpy(a,b,c)
#define scmchr(a,b,c)     _fmemchr(a,b,c)
#define scmcmp(a,b,c)     _fmemcmp(a,b,c)
#define scmset(a,b,c)     _fmemset(a,b,c)
#else
#define ULEN UINT
#define M_HNDL VOID F_PTR
#define scslen(a)         strlen(a)
#define scscmp(a,b)       strcmp(a,b)
#define scsicmp(a,b)      strcmpi(a,b)
#define scsrchr(a,b)      strrchr(a,b)
#define scsstr(a,b)       strstr(a,b)
#define scmcpy(a,b,c)     memcpy(a,b,c)
#define scmchr(a,b,c)     memchr(a,b,c)
#define scmcmp(a,b,c)     memcmp(a,b,c)
#define scmset(a,b,c)     memset(a,b,c)
#endif





/**********
 *
 *  Miscellaneous Prototypes
 *
 **********/



#ifdef __ANSI_PROTOTYPES__
#ifdef __cplusplus
extern "C" {
#endif

#define scmalloc(a,b) _scmalloc(a,b)
INT PROTOS _scmalloc(  M_HNDL F_PTR,     /* memory address/handle */
		       ULEN);            /* number of bytes to allocate */

#define scmralloc(a,b) _scmralloc(a,b)
INT PROTOS _scmralloc( M_HNDL F_PTR,     /* existing memory address/handle */
		       ULEN);            /* new size in bytes */

#define scmhfree(a) _scmfree(a)
INT PROTOS _scmfree(   M_HNDL);          /* free memory by handle */

INT PROTOS scmfree(    CHAR F_PTR );     /* free memory by address */

#ifdef _Windows
#define scmlock(a,b) _scmlock(a,b)
INT PROTOS _scmlock(   CHAR F_PTR F_PTR, /* memory address */
		       M_HNDL);          /* memory handle */

#define scmunlock(a) _scmunlock(a)
INT PROTOS _scmunlock( M_HNDL);          /* memory handle/address */
#else
#define scmlock(a,b)
#define _scmlock(a,b)
#define scmunlock(a)
#define _scmunlock(a)
#endif

#ifdef __cplusplus
}
#endif
#endif


#endif


